@@ -224,9 +224,9 @@ package, it is really a C<request_rec*> in disguise.
 
 =over 4
 
-=item Apache->request([$r])
+=item Apache-E<gt>request([$r])
 
-The Apache->request method will return a reference to the request object.
+The Apache-E<gt>request method will return a reference to the request object.
 
 B<Perl*Handler>s can obtain a reference to the request object when it
 is passed to them via C<@_>.  However, scripts that run under 
@@ -236,43 +236,43 @@ by passing an object reference to C<Apache-E<gt>request($r)>.
 If handlers use modules such as B<CGI::Apache> that need to access
 C<Apache-E<gt>request>, they too should do this (e.g. B<Apache::Status>).
 
-=item $r->as_string
+=item $r-E<gt>as_string
 
 Returns a string representation of the request object.  Mainly useful
 for debugging.
 
-=item $r->main
+=item $r-E<gt>main
 
 If the current request is a sub-request, this method returns a blessed
 reference to the main request structure.  If the current request is
 the main request, then this method returns C<undef>.
 
-=item $r->prev
+=item $r-E<gt>prev
 
 This method returns a blessed reference to the previous (internal) request
 structure or C<undef> if there is no previous request.
 
-=item $r->next
+=item $r-E<gt>next
 
 This method returns a blessed reference to the next (internal) request
 structure or C<undef> if there is no next request.
 
-=item $r->last
+=item $r-E<gt>last
 
 This method returns a blessed reference to the last (internal) request
 structure.  Handy for logging modules.
 
-=item $r->is_main
+=item $r-E<gt>is_main
 
 Returns true if the current request object is for the main request.
 (Should give the same result as C<!$r-E<gt>main>, but will be more efficient.)
 
-=item $r->is_initial_req
+=item $r-E<gt>is_initial_req
 
 Returns true if the current request is the first internal request,
 returns false if the request is a sub-request or internal redirect.
 
-=item $r->allowed($bitmask)
+=item $r-E<gt>allowed($bitmask)
 
 Get or set the allowed methods bitmask. This allowed bitmask should be
 set whenever a 405 (method not allowed) or 501 (method not implemented)
@@ -300,7 +300,7 @@ B<Apache> class.
 
 =over 4
 
-=item $r->lookup_uri($uri)
+=item $r-E<gt>lookup_uri($uri)
 
    my $subr = $r->lookup_uri($uri);
    my $filename = $subr->filename;
@@ -309,11 +309,11 @@ B<Apache> class.
        warn "can't stat $filename!\n";
    } 
 
-=item $r->lookup_file($filename)
+=item $r-E<gt>lookup_file($filename)
 
    my $subr = $r->lookup_file($filename);
 
-=item $subr->run
+=item $subr-E<gt>run
 
    if($subr->run != OK) {
        $subr->log_error("something went wrong!");
@@ -331,113 +331,119 @@ or I<Apache-E<gt>request>
 
 =over 4
 
-=item $r->method( [$meth] )
+=item $r-E<gt>method( [$meth] )
 
-The $r->method method will return the request method.  It will be a
+The $r-E<gt>method method will return the request method.  It will be a
 string such as "GET", "HEAD" or "POST".
 Passing an argument will set the method, mainly used for internal redirects.
 
-=item $r->method_number( [$num] )
+=item $r-E<gt>method_number( [$num] )
 
-The $r->method_number method will return the request method number.
+The $r-E<gt>method_number method will return the request method number.
 The method numbers are defined by the M_GET, M_POST,... constants
 available from the B<Apache::Constants> module.  Passing an argument
 will set the method_number, mainly used for internal redirects and
 testing authorization restriction masks.
 
-=item $r->bytes_sent
+=item $r-E<gt>bytes_sent
 
 The number of bytes sent to the client, handy for logging, etc.
 
-=item $r->the_request
+=item $r-E<gt>the_request
 
 The request line sent by the client, handy for logging, etc.
 
-=item $r->proxyreq
+=item $r-E<gt>proxyreq
 
 Returns true if the request is proxy http.
 Mainly used during the filename translation stage of the request, 
 which may be handled by a C<PerlTransHandler>.
 
-=item $r->header_only
+=item $r-E<gt>header_only
 
 Returns true if the client is asking for headers only, 
 e.g. if the request method was B<HEAD>.
 
-=item $r->protocol
+=item $r-E<gt>protocol
 
-The $r->protocol method will return a string identifying the protocol
+The $r-E<gt>protocol method will return a string identifying the protocol
 that the client speaks.  Typical values will be "HTTP/1.0" or
 "HTTP/1.1".
 
-=item $r->hostname
+=item $r-E<gt>hostname
 
 Returns the server host name, as set by full URI or Host: header.
 
-=item $r->request_time
+=item $r-E<gt>request_time
 
 Returns the time that the request was made.  The time is the local unix
 time in seconds since the epoch.
 
-=item $r->uri( [$uri] )
+=item $r-E<gt>uri( [$uri] )
 
-The $r->uri method will return the requested URI minus optional query
+The $r-E<gt>uri method will return the requested URI minus optional query
 string, optionally changing it with the first argument.
 
-=item $r->filename( [$filename] )
+=item $r-E<gt>filename( [$filename] )
 
-The $r->filename method will return the result of the I<URI --E<gt>
+The $r-E<gt>filename method will return the result of the I<URI --E<gt>
 filename> translation, optionally changing it with the first argument
 if you happen to be doing the translation.
 
-=item $r->path_info( [$path_info] )
+=item $r-E<gt>location
 
-The $r->path_info method will return what is left in the path after the
+The $r-E<gt>location method will return the path of the
+E<lt>LocationE<gt> section from which the current C<Perl*Handler> is
+being called.
+
+=item $r-E<gt>path_info( [$path_info] )
+
+The $r-E<gt>path_info method will return what is left in the path after the
 I<URI --E<gt> filename> translation, optionally changing it with the first 
 argument if you happen to be doing the translation.
 
-=item $r->args( [$query_string] )
+=item $r-E<gt>args( [$query_string] )
 
-The $r->args method will return the contents of the URI I<query
+The $r-E<gt>args method will return the contents of the URI I<query
 string>.  When called in a scalar context, the entire string is
-returned.  When called in a list context, a list of parsed I<key> =>
+returned.  When called in a list context, a list of parsed I<key> =E<gt>
 I<value> pairs are returned, i.e. it can be used like this:
 
    $query = $r->args;
    %in    = $r->args;
 
-$r->args can also be used to set the I<query string>. This can be useful
+$r-E<gt>args can also be used to set the I<query string>. This can be useful
 when redirecting a POST request.
 
-=item $r->headers_in
+=item $r-E<gt>headers_in
 
-The $r->headers_in method will return a %hash of client request
+The $r-E<gt>headers_in method will return a %hash of client request
 headers.  This can be used to initialize a perl hash, or one could use
-the $r->header_in() method (described below) to retrieve a specific
+the $r-E<gt>header_in() method (described below) to retrieve a specific
 header value directly.
 
 Will return a I<HASH> reference blessed into the I<Apache::Table>
 class when called in a scalar context with no "key" argument. This
 requires I<Apache::Table>.
 
-=item $r->header_in( $header_name, [$value] )
+=item $r-E<gt>header_in( $header_name, [$value] )
 
 Return the value of a client header.  Can be used like this:
 
    $ct = $r->header_in("Content-type");
    $r->header_in($key, $val); #set the value of header '$key'
 
-=item $r->content
+=item $r-E<gt>content
 
-The $r->content method will return the entity body read from the
+The $r-E<gt>content method will return the entity body read from the
 client, but only if the request content type is
 C<application/x-www-form-urlencoded>.
 When called in a scalar context, the entire string is
-returned.  When called in a list context, a list of parsed I<key> =>
+returned.  When called in a list context, a list of parsed I<key> =E<gt>
 I<value> pairs are returned.  *NOTE*: you can only ask for this once,
 as the entire body is read from the client.
 
-=item $r->read($buf, $bytes_to_read, [$offset])
+=item $r-E<gt>read($buf, $bytes_to_read, [$offset])
 
 This method is used to read data from the client, 
 looping until it gets all of C<$bytes_to_read> or a timeout happens.
@@ -448,14 +454,14 @@ than the beginning of the string.
 In addition, this method sets a timeout before reading with
 C<$r-E<gt>soft_timeout>.
 
-=item $r->get_remote_host
+=item $r-E<gt>get_remote_host
 
 Lookup the client's DNS hostname. If the configuration directive
 B<HostNameLookups> is set to off, this returns the dotted decimal
 representation of the client's IP address instead. Might return
 I<undef> if the hostname is not known.
 
-=item $r->get_remote_logname
+=item $r-E<gt>get_remote_logname
 
 Lookup the remote user's system name.  Might return I<undef> if the
 remote system is not running an RFC 1413 server or if the configuration
@@ -468,16 +474,16 @@ B<Apache::Connection> object, as described below.
 
 =over 4
 
-=item $c = $r->connection
+=item $c = $r-E<gt>connection
 
-The $r->connection method will return a reference to the request
+The $r-E<gt>connection method will return a reference to the request
 connection object (blessed into the B<Apache::Connection> package).
 This is really a C<conn_rec*> in disguise.  The following methods can
 be used on the connection object:
 
 =over 4
 
-=item $c->remote_host
+=item $c-E<gt>remote_host
 
 If the configuration directive B<HostNameLookups> is set to on:  then
 the first time C<$r-E<gt>get_remote_host> is called the server does a DNS
@@ -493,7 +499,7 @@ string is not cached, and C<$c-E<gt>remote_host> is undefined. So, it's
 best to to call C<$r-E<gt>get_remote_host> instead of directly accessing
 this variable.
 
-=item $c->remote_ip
+=item $c-E<gt>remote_ip
 
 The dotted decimal representation of the remote client's IP address.
 This is set by the server when the connection record is created so
@@ -503,7 +509,7 @@ You can also set this value by providing an argument to it. This is
 helpful if your server is behind a squid accelerator proxy which adds
 a X-Forwarded-For header.
 
-=item $c->local_addr
+=item $c-E<gt>local_addr
 
 A packed SOCKADDR_IN in the same format as returned by
 L<Socket/pack_sockaddr_in>, containing the port and address on the
@@ -511,7 +517,7 @@ local host that the remote client is connected to.  This is set by
 the server when the connection record is created so it is always
 defined.
 
-=item $c->remote_addr
+=item $c-E<gt>remote_addr
 
 A packed SOCKADDR_IN in the same format as returned by
 L<Socket/pack_sockaddr_in>, containing the port and address on the
@@ -533,7 +539,7 @@ Note that the lookupFromInAddr interface does not currently exist in
 the B<Net::Ident> module, but the author is planning on adding it
 soon.
 
-=item $c->remote_logname
+=item $c-E<gt>remote_logname
 
 If the configuration directive B<IdentityCheck> is set to on:  then the
 first time C<$r-E<gt>get_remote_logname> is called the server does an RFC
@@ -546,22 +552,22 @@ If the configuration directive B<IdentityCheck> is set to off: then
 C<$r-E<gt>get_remote_logname> does nothing and C<$c-E<gt>remote_logname> is
 always undefined.
 
-=item $c->user( [$user] )
+=item $c-E<gt>user( [$user] )
 
 If an authentication check was successful, the authentication handler
 caches the user name here. Sets the user name to the optional first
 argument.
 
-=item $c->auth_type
+=item $c-E<gt>auth_type
 
 Returns the authentication scheme that successfully authenticate
 C<$c-E<gt>user>, if any.
 
-=item $c->aborted
+=item $c-E<gt>aborted
 
 Returns true if the client stopped talking to us.
 
-=item $c->fileno( [$direction] )
+=item $c-E<gt>fileno( [$direction] )
 
 Returns the client file descriptor. If $direction is 0, the input fd
 is returned. If $direction is not null or ommitted, the output fd is
@@ -581,7 +587,7 @@ configuration and access control files.
 
 =over 4
 
-=item $r->dir_config( $key )
+=item $r-E<gt>dir_config( $key )
 
 Returns the value of a per-directory variable specified by the 
 C<PerlSetVar> directive.
@@ -599,7 +605,7 @@ I<Apache::Table> class when called in a scalar context with no
 "key" argument. See I<Apache::Table>.
 
 
-=item $r->dir_config->get( $key )
+=item $r-E<gt>dir_config-E<gt>get( $key )
 
 Returns the value of a per-directory array variable specified by the
 C<PerlAddVar> directive.
@@ -621,13 +627,13 @@ Will return a I<HASH> reference blessed into the
 I<Apache::Table> class when called in a scalar context with no
 "key" argument. See I<Apache::Table>.
 
-=item $r->requires
+=item $r-E<gt>requires
 
 Returns an array reference of hash references, containing information
 related to the B<require> directive.  This is normally used for access
 control, see L<Apache::AuthzAge> for an example.
 
-=item $r->auth_type
+=item $r-E<gt>auth_type
 
 Returns a reference to the current value of the per directory
 configuration directive B<AuthType>. Normally this would be set to
@@ -635,7 +641,7 @@ C<Basic> to use the basic authentication scheme defined in RFC 1945,
 I<Hypertext Transfer Protocol -- HTTP/1.0>. However, you could set to
 something else and implement your own authentication scheme.
 
-=item $r->auth_name
+=item $r-E<gt>auth_name
 
 Returns a reference to the current value of the per directory
 configuration directive B<AuthName>.  The AuthName directive creates
@@ -650,16 +656,30 @@ Then during the authentication stage the server uses the current
 authentication realm, from C<$r-E<gt>auth_name>, to determine which set of
 credentials to authenticate.
 
-=item $r->document_root
+=item $r-E<gt>document_root ( [$docroot] )
 
-Returns a reference to the current value of the per server
-configuration directive B<DocumentRoot>. To quote the Apache server
-documentation, "Unless matched by a directive like Alias, the server
-appends the path from the requested URL to the document root to make
-the path to the document."  This same value is passed to CGI
+When called with no argument, returns a reference to the current value
+of the per server configuration directive B<DocumentRoot>. To quote the
+Apache server documentation, "Unless matched by a directive like Alias,
+the server appends the path from the requested URL to the document root
+to make the path to the document."  This same value is passed to CGI
 scripts in the C<DOCUMENT_ROOT> environment variable.
 
-=item $r->allow_options
+You can also set this value by providing an argument to it. The following
+example dynamically sets the document root based on the request's
+"Host:" header:
+
+   sub trans_handler
+     {
+        my $r = shift;
+        my ($user) = ($r->header_in('Host') =~ /^[^\.]+/);
+        $r->document_root("/home/$user/www");
+        return DECLINED;
+     }
+    
+   PerlTransHandler trans_handler
+
+=item $r-E<gt>allow_options
 
 The C<$r-E<gt>allow_options> method can be used for
 checking if it is OK to run a perl script.  The B<Apache::Options>
@@ -670,64 +690,64 @@ module provides the constants to check against.
 		      $filename);
    }
 
-=item $r->get_server_port
+=item $r-E<gt>get_server_port
 
 Returns the port number on which the server is listening.
 
-=item $s = $r->server
+=item $s = $r-E<gt>server
 
 Return a reference to the server info object (blessed into the
 B<Apache::Server> package).  This is really a C<server_rec*> in
 disguise.  The following methods can be used on the server object:
 
-=item $s = Apache->server
+=item $s = Apache-E<gt>server
 
 Same as above, but only available during server startup for use in
-<Perl> sections, B<PerlScript> or B<PerlModule>.
+C<E<lt>PerlE<gt>> sections, B<PerlRequire> or B<PerlModule>.
 
-=item $s->server_admin
+=item $s-E<gt>server_admin
 
 Returns the mail address of the person responsible for this server.
 
-=item $s->server_hostname
+=item $s-E<gt>server_hostname
 
 Returns the hostname used by this server.
 
-=item $s->port
+=item $s-E<gt>port
 
 Returns the port that this servers listens too.
 
-=item $s->is_virtual
+=item $s-E<gt>is_virtual
 
 Returns true if this is a virtual server.
 
-=item $s->names
+=item $s-E<gt>names
 
 Returns the wild-carded names for ServerAlias servers. 
 
-=item $s->dir_config( $key )
+=item $s-E<gt>dir_config( $key )
 
 Alias for Apache::dir_config.
 
-=item $s->warn
+=item $s-E<gt>warn
 
 Alias for Apache::warn.
 
-=item $s->log_error
+=item $s-E<gt>log_error
 
 Alias for Apache::log_error.
 
-=item $s->uid
+=item $s-E<gt>uid
 
 Returns the numeric user id under which the server answers requests.
 This is the value of the User directive.
 
-=item $s->gid
+=item $s-E<gt>gid
 
 Returns the numeric group id under which the server answers requests.
 This is the value of the Group directive.
 
-=item $s->loglevel
+=item $s-E<gt>loglevel
 
 Get or set the value of the current LogLevel. This method is added by
 the Apache::Log module, which needs to be pulled in.
@@ -748,7 +768,7 @@ exported):
     Apache::Log::INFO
     Apache::Log::DEBUG
 
-=item $r->get_handlers( $hook )
+=item $r-E<gt>get_handlers( $hook )
 
 Returns a reference to a list of handlers enabled for $hook. $hook is
 a string representing the phase to handle. The returned list is a list
@@ -756,73 +776,83 @@ of references to the handler subroutines.
 
 	$list = $r->get_handlers( 'PerlHandler' );
 
-=item $r->set_handlers( $hook, [\&handler, ... ] )
+=item $r-E<gt>set_handlers( $hook, [\E<amp>handler, ... ] )
 
 Sets the list if handlers to be called for $hook. $hook is a string
 representing the phase to handle. The list of handlers is an anonymous
 array of code references to the handlers to install for this request
-phase. The special list [ \&OK ] can be used to disable a particular
-phase.
+phase. The special list C<[ \&OK ]> can be used to disable a
+particular phase.
 
 	$r->set_handlers( PerlLogHandler => [ \&myhandler1, \&myhandler2 ] );
 	$r->set_handlers( PerlAuthenHandler => [ \&OK ] );
 
-=item $r->push_handlers( $hook, \&handler )
+=item $r-E<gt>push_handlers( $hook, \E<amp>handler )
 
-Pushes a new handler to be called for $hook. $hook is a string
+Pushes a new handler to be called for C<$hook>. C<$hook> is a string
 representing the phase to handle. The handler is a reference to a
 subroutine to install for this request phase. This handler will be
 called before any configured handlers.
 
 	$r->push_handlers( PerlHandler => \&footer);
 
+=item $r-E<gt>current_callback
+
+Returns the name of the handler currently being run. This method is most
+useful to PerlDispatchHandlers who wish to only take action for certain phases.
+
+	if($r->current_callback eq "PerlLogHandler") {
+		$r->warn("Logging request");
+	}
+
 =back
 
 =head1 SETTING UP THE RESPONSE
 
 The following methods are used to set up and return the response back
-to the client.  This typically involves setting up $r->status(), the
+to the client.  This typically involves setting up $r-E<gt>status(), the
 various content attributes and optionally some additional
-$r->header_out() calls before calling $r->send_http_header() which will
+$r-E<gt>header_out() calls before calling $r-E<gt>send_http_header() which will
 actually send the headers to the client.  After this a typical
-application will call the $r->print() method to send the response
+application will call the $r-E<gt>print() method to send the response
 content to the client.
 
 =over 4
 
-=item $r->send_http_header( [$content_type] )
+=item $r-E<gt>send_http_header( [$content_type] )
 
 Send the response line and all headers to the client.
 Takes an optional parameter indicating the content-type of the
 response, i.e. 'text/html'.
 
-This method will create headers from the $r->content_xxx() and
-$r->no_cache() attributes (described below) and then append the
-headers defined by $r->header_out (or $r->err_header_out if status
+This method will create headers from the $r-E<gt>content_xxx() and
+$r-E<gt>no_cache() attributes (described below) and then append the
+headers defined by $r-E<gt>header_out (or $r-E<gt>err_header_out if status
 indicates an error).
 
-=item $r->get_basic_auth_pw
+=item $r-E<gt>get_basic_auth_pw
 
-If the current request is protected by Basic authentication, 
-this method will return 0, otherwise -1.  
+If the current request is protected by Basic authentication, this
+method will return OK.  Otherwise, it will return a value that
+ought to be propagated back to the client (typically AUTH_REQUIRED).
 The second return value will be the decoded password sent by the client.
 
    ($ret, $sent_pw) = $r->get_basic_auth_pw;
 
-=item $r->note_basic_auth_failure
+=item $r-E<gt>note_basic_auth_failure
 
 Prior to requiring Basic authentication from the client, this method 
 will set the outgoing HTTP headers asking the client to authenticate 
 for the realm defined by the configuration directive C<AuthName>.
 
-=item $r->handler( [$meth] )
+=item $r-E<gt>handler( [$meth] )
 
 Set the handler for a request.
 Normally set by the configuration directive C<AddHandler>.
 
    $r->handler( "perl-script" );
 
-=item $r->notes( $key, [$value] )
+=item $r-E<gt>notes( $key, [$value] )
 
 Return the value of a named entry in the Apache C<notes> table, or
 optionally set the value of a named entry.  This table is used by Apache
@@ -836,18 +866,18 @@ Will return a I<HASH> reference blessed into the I<Apache::Table>
 class when called in a scalar context with no "key" argument. This
 requires I<Apache::Table>.
 
-=item $r->pnotes( $key, [$value] )
+=item $r-E<gt>pnotes( $key, [$value] )
 
-Like $r->notes, but takes any scalar as an value.
+Like $r-E<gt>notes, but takes any scalar as an value.
 
    $r->pnotes("MY_HANDLER" => [qw(one two)]);
    my $val = $r->pnotes("MY_HANDLER");
    print $val->[0];     # prints "one"
 
-Advantage over just using a Perl variable is that $r->pnotes gets
+Advantage over just using a Perl variable is that $r-E<gt>pnotes gets
 cleaned up after every request.
 
-=item $r->subprocess_env( $key, [$value] )
+=item $r-E<gt>subprocess_env( $key, [$value] )
 
 Return the value of a named entry in the Apache C<subprocess_env>
 table, or optionally set the value of a named entry. This table is
@@ -871,7 +901,7 @@ Will return a I<HASH> reference blessed into the I<Apache::Table>
 class when called in a scalar context with no "key" argument. This
 requires I<Apache::Table>.
 
-=item $r->content_type( [$newval] )
+=item $r-E<gt>content_type( [$newval] )
 
 Get or set the content type being sent to the client.  Content types
 are strings like "text/plain", "text/html" or "image/gif".  This
@@ -881,42 +911,42 @@ of usage is:
    $previous_type = $r->content_type;
    $r->content_type("text/plain");
 
-=item $r->content_encoding( [$newval] )
+=item $r-E<gt>content_encoding( [$newval] )
 
 Get or set the content encoding.  Content encodings are string like
 "gzip" or "compress".  This correspond to the "Content-Encoding"
 header in the HTTP protocol.
 
-=item $r->content_languages( [$array_ref] )
+=item $r-E<gt>content_languages( [$array_ref] )
 
 Get or set the content languages.  The content language corresponds to the
 "Content-Language" HTTP header and is an array reference containing strings
 such as "en" or "no".
 
-=item $r->status( $integer )
+=item $r-E<gt>status( $integer )
 
 Get or set the reply status for the client request.  The
 B<Apache::Constants> module provide mnemonic names for the status codes.
 
-=item $r->status_line( $string )
+=item $r-E<gt>status_line( $string )
 
 Get or set the response status line.  The status line is a string like
 "200 Document follows" and it will take precedence over the value specified
-using the $r->status() described above.
+using the $r-E<gt>status() described above.
 
 
-=item $r->headers_out
+=item $r-E<gt>headers_out
 
-The $r->headers_out method will return a %hash of server response
+The $r-E<gt>headers_out method will return a %hash of server response
 headers.  This can be used to initialize a perl hash, or one could use
-the $r->header_out() method (described below) to retrieve or set a specific
+the $r-E<gt>header_out() method (described below) to retrieve or set a specific
 header value directly.
 
 Will return a I<HASH> reference blessed into the I<Apache::Table>
 class when called in a scalar context with no "key" argument. This
 requires I<Apache::Table>.
 
-=item $r->header_out( $header, $value )
+=item $r-E<gt>header_out( $header, $value )
 
 Change the value of a response header, or create a new one.  You
 should not define any "Content-XXX" headers by calling this method,
@@ -925,11 +955,11 @@ because these headers use their own specific methods.  Example of use:
    $r->header_out("WWW-Authenticate" => "Basic");
    $val = $r->header_out($key);
 
-=item $r->err_headers_out
+=item $r-E<gt>err_headers_out
 
-The $r->err_headers_out method will return a %hash of server response
+The $r-E<gt>err_headers_out method will return a %hash of server response
 headers.  This can be used to initialize a perl hash, or one could use
-the $r->err_header_out() method (described below) to retrieve or set a specific
+the $r-E<gt>err_header_out() method (described below) to retrieve or set a specific
 header value directly.
 
 The difference between headers_out and err_headers_out is that the
@@ -940,7 +970,7 @@ Will return a I<HASH> reference blessed into the I<Apache::Table>
 class when called in a scalar context with no "key" argument. This
 requires I<Apache::Table>.
 
-=item $r->err_header_out( $header, [$value] )
+=item $r-E<gt>err_header_out( $header, [$value] )
 
 Change the value of an error response header, or create a new one.
 These headers are used if the status indicates an error.
@@ -948,12 +978,14 @@ These headers are used if the status indicates an error.
    $r->err_header_out("Warning" => "Bad luck");
    $val = $r->err_header_out($key);
 
-=item $r->no_cache( $boolean )
+=item $r-E<gt>no_cache( $boolean )
 
 This is a flag that indicates that the data being returned is volatile
-and the client should be told not to cache it.
+and the client should be told not to cache it. C<$r-E<gt>no_cache(1)>
+adds the headers "Pragma: no-cache" and "Cache-control: no-cache" to
+the reponse, therefore it must be called before C<$r-E<gt>send_http_header>.
 
-=item $r->print( @list )
+=item $r-E<gt>print( @list )
 
 This method sends data to the client with C<$r-E<gt>write_client>, but first
 sets a timeout before sending with C<$r-E<gt>soft_timeout>. This method is
@@ -972,7 +1004,7 @@ The result is "bar", not the "SCALAR(0xDEADBEEF)" you might have expected. If
 you really want the reference to be printed out, force it into a scalar
 context by using C<print(scalar($foo))>.
 
-=item $r->send_fd( $filehandle )
+=item $r-E<gt>send_fd( $filehandle )
 
 Send the contents of a file to the client.  Can for instance be used
 like this:
@@ -981,18 +1013,18 @@ like this:
   $r->send_fd(FILE);
   close(FILE);
 
-=item $r->internal_redirect( $newplace )
+=item $r-E<gt>internal_redirect( $newplace )
 
 Redirect to a location in the server namespace without 
 telling the client. For instance:
 
    $r->internal_redirect("/home/sweet/home.html");
 
-=item $r->internal_redirect_handler( $newplace )
+=item $r-E<gt>internal_redirect_handler( $newplace )
 
 Same as I<internal_redirect>, but the I<handler> from C<$r> is preserved.
 
-=item $r->custom_response($code, $uri)
+=item $r-E<gt>custom_response($code, $uri)
 
 This method provides a hook into the B<ErrorDocument> mechanism,
 allowing you to configure a custom response for a given response
@@ -1031,13 +1063,13 @@ Example:
 
 =over 4
 
-=item $r->soft_timeout($message)
+=item $r-E<gt>soft_timeout($message)
 
-=item $r->hard_timeout($message)
+=item $r-E<gt>hard_timeout($message)
 
-=item $r->kill_timeout
+=item $r-E<gt>kill_timeout
 
-=item $r->reset_timeout
+=item $r-E<gt>reset_timeout
 
 (Documentation borrowed from http_main.h)
 
@@ -1072,11 +1104,11 @@ kill_timeout() will disarm either variety of timeout.
 
 reset_timeout() resets the timeout in progress.
 
-=item $r->post_connection($code_ref)
+=item $r-E<gt>post_connection($code_ref)
 
-=item $r->register_cleanup($code_ref)
+=item $r-E<gt>register_cleanup($code_ref)
 
-Register a cleanup function which is called just before $r->pool is
+Register a cleanup function which is called just before $r-E<gt>pool is
 destroyed.
 
    $r->register_cleanup(sub {
@@ -1104,11 +1136,11 @@ type of interface.
 
 =over 4
 
-=item $r->send_cgi_header()
+=item $r-E<gt>send_cgi_header()
 
 Take action on certain headers including I<Status:>, I<Location:> and
 I<Content-type:> just as mod_cgi does, then calls
-$r->send_http_header().  Example of use:
+$r-E<gt>send_http_header().  Example of use:
 
    $r->send_cgi_header(<<EOT);
    Location: /foo/bar
@@ -1124,19 +1156,19 @@ The following methods can be used to log errors.
 
 =over 4
 
-=item $r->log_reason($message, $file)
+=item $r-E<gt>log_reason($message, $file)
 
 The request failed, why??  Write a message to the server errorlog.
 
    $r->log_reason("Because I felt like it", $r->filename);
 
-=item $r->log_error($message)
+=item $r-E<gt>log_error($message)
 
 Uh, oh.  Write a message to the server errorlog.
 
    $r->log_error("Some text that goes in the error_log");
 
-=item $r->warn($message)
+=item $r-E<gt>warn($message)
 
 For pre-1.3 versions of apache, this is just an alias for
 C<log_error>.  With 1.3+ versions of apache, this message will only be
