@@ -1,6 +1,6 @@
 package Apache::Log;
 
-use Apache ();
+use mod_perl ();
 
 $VERSION = '1.01';
 __PACKAGE__->mod_perl::boot($VERSION);
