@@ -1,38 +1,61 @@
 package Apache::StatINC;
-
 use strict;
 
-$Apache::StatINC::VERSION = "1.06";
+$Apache::StatINC::VERSION = "1.07";
 
 my %Stat = ($INC{"Apache/StatINC.pm"} => time);
 
 sub handler {
     my $r = shift;
-    my $do_undef = ref($r) && (lc($r->dir_config("UndefOnReload") || '') eq "on");
+    my $do_undef = ref($r) && ((lc($r->dir_config("StatINC_UndefOnReload") ||
+				   $r->dir_config("UndefOnReload")) || '') eq "on");
     my $DEBUG = ref($r) && (lc($r->dir_config("StatINCDebug") || '') eq "on");
+    $DEBUG = $r->dir_config("StatINC_Debug") if ref($r) && $r->dir_config("StatINC_Debug");
 
     while(my($key,$file) = each %INC) {
 	local $^W = 0;
 	my $mtime = (stat $file)[9];
-	  # warn and skip the files with relative paths which can't be locate by applying @INC;
-	warn "Apache::StatINC: Can't locate $file\n",next unless defined $mtime and $mtime;
+	# warn and skip the files with relative paths which can't be locate by applying @INC;
+	warn("Apache::StatINC: Can't locate $file\n"),next unless defined $mtime and $mtime;
 	unless(defined $Stat{$file}) { 
 	    $Stat{$file} = $^T;
 	}
+        # if modified, reload the module
 	if($mtime > $Stat{$file}) {
+	    # make sure file's prefix is in @INC
+	    my $found_in_inc;
+	    for (@INC) {
+	       if(index($file,$_) == 0) {
+	          $found_in_inc = 1;
+	          last;
+	       }
+	    }
+
+            if(!$found_in_inc) {
+               my $inc_dir = substr($file, 0, length($file)-length($key)-1);
+               push @INC, $inc_dir;
+	       warn "Apache::StatINC: process $$ adding $inc_dir to \@INC\n"
+                   if $DEBUG > 0;
+            }
+
 	    if($do_undef and $key =~ /\.pm$/) {
 		require Apache::Symbol;
 		my $class = Apache::Symbol::file2class($key);
-               $class->Apache::Symbol::undef_functions( undef, 1 );
+		$class->Apache::Symbol::undef_functions( undef, 1 );
 	    }
 	    delete $INC{$key};
-	    require $key;
-	    warn "Apache::StatINC: process $$ reloading $key\n"
-		  if $DEBUG;
+	    eval{ require $key };
+	    if ($@) {
+               warn "Apache::StatINC: process $$ failed to reload $key. $@"
+                  if $DEBUG > 0;
+	    } else {
+	       warn "Apache::StatINC: process $$ reloading $key.\n"
+                  if $DEBUG > 0;
+            }
 	}
 	$Stat{$file} = $mtime;
     }
-
+    
     return 1;
 }
 
@@ -88,7 +111,7 @@ It will most likely help you to find the problem. Really.
 
 =over 4
 
-=item UndefOnReload
+=item StatINC_UndefOnReload
 
 Normally, StatINC will turn of warnings to avoid "Subroutine redefined" 
 warnings when it reloads a file.  However, this does not disable the 
@@ -97,14 +120,16 @@ Perl mandatory warning when re-defining C<constant> subroutines
 B<Apache::Symbol> I<undef_functions> method to avoid these mandatory
 warnings:
 
- PerlSetVar UndefOnReload On
+ PerlSetVar StatINC_UndefOnReload On
 
-=item StatINCDebug
+=item StatINC_Debug
 
 You can make StatINC tell when it reloads a module by setting this
 option to on.
 
- PerlSetVar StatINCDebug On
+ PerlSetVar StatINC_Debug 1
+
+The only used debug level is currently 1. 
 
 =back
 
@@ -114,7 +139,7 @@ mod_perl(3)
 
 =head1 AUTHOR
 
-Currently maintained by Ask Bjoern Hansen.
+Currently maintained by Ask Bjoern Hansen <ask@netcetera.dk>.
 Written by Doug MacEachern.
 
 
