@@ -91,7 +91,7 @@ sub new {
 
 sub mmn_eq {
     my($class, $dir) = @_;
-    return 1 if $Is_Win32; #just assume, till Apache::src works under win32 
+#    return 1 if $Is_Win32; #just assume, till Apache::src works under win32 
     my $instsrc;
     {
 	local @INC = grep { !/blib/ } @INC;
@@ -212,8 +212,11 @@ sub httpd_version {
 
     while(<$fh>) {
 	next unless /^#define/;
-	s/SERVER_PRODUCT \"/\"Apache/; #1.3.13+
-	next unless s/^#define\s+SERVER_(BASE|)VERSION\s+"(.*)\s*".*/$2/;
+	s/SERVER_PRODUCT \"/\"Apache/; #1.3.13
+	next unless s/^#define\s+SERVER_(BASE|)(VERSION|REVISION)\s+"(.*)\s*".*/$3/;
+        unless (m:/:) {
+            $_ = "Apache/$_"; #1.3.14, argh
+        }
 	chomp($string = $_);
 
 	#print STDERR "Examining SERVER_VERSION '$string'...";
@@ -255,7 +258,12 @@ sub otherldflags {
 	    push @ldflags, "-bI:" . $file;
 	}
 	my $httpdexp = $self->apxs("-q" => 'LIBEXECDIR') . "/httpd.exp";
-	push @ldflags, "-bI:$httpdexp" if -e $httpdexp;
+	if (-e $httpdexp) {
+		push @ldflags, "-bI:$httpdexp";
+	} else {
+		$httpdexp = $self->dir . "/support/httpd.exp";
+		push @ldflags, "-bI:$httpdexp" if -e $httpdexp;
+	}
     }
     return join(' ', @ldflags);
 }
