@@ -12,6 +12,8 @@ use FileHandle ();
 @EXPORT_OK = qw(have_httpd);
 
 BEGIN { 
+    $ENV{PERL_LWP_USE_HTTP_10} = 1; #default to http/1.0
+
     if(not $ENV{MOD_PERL}) {
 	eval { require "net/config.pl"; }; #for 'make test'
 	$PERL_DIR = $net::perldir;
@@ -37,6 +39,7 @@ my $UA;
 
 eval {
     require LWP::UserAgent;
+    require URI::URL;
     $UA = LWP::UserAgent->new;
 };
 
@@ -517,7 +520,7 @@ __END__
 
 =head1 NAME
 
-Apache::Test - Facilitates testing of Apache::* modules
+Apache::test - Facilitates testing of Apache::* modules
 
 =head1 SYNOPSIS
 
