@@ -1,6 +1,6 @@
 package Bundle::Apache;
 
-$VERSION = '1.01';
+$VERSION = '1.02';
 
 1;
 
@@ -16,17 +16,17 @@ C<perl -MCPAN -e 'install Bundle::Apache'>
 
 =head1 CONTENTS
 
-Apache - Perl interface to Apache server API
+MIME::Base64       - Used in authentication headers
 
-ExtUtils::Embed - Needed to build httpd
+Digest::MD5        - Needed to do Digest authentication
 
-MIME::Base64 - Needed for LWP
+URI 0.10           - There are URIs everywhere
 
-URI::URL - Needed for LWP
+Net::FTP 2.00      - If you want ftp://-support
 
-LWP - Web client to run mod_perl tests
+HTML::HeadParser   - To get the correct $res->base
 
-HTML::TreeBuilder - Used for Apache::SSI
+LWP                - The reason why you need the modules above
 
 Devel::Symdump - Symbol table browsing with Apache::Status
 
@@ -36,14 +36,18 @@ CGI - CGI.pm
 
 Tie::IxHash - For order in <Perl> sections
 
-Apache::ePerl - mod_perl adaptation of 'ePerl'
+Apache - Perl interface to Apache server API
 
 Apache::DBI   - Wrapper around DBI->connect to transparently maintain persistent connections
 
+Apache::DB - Run the interactive Perl debugger under mod_perl
+
 Apache::Stage - Management of document staging directories
 
 Apache::Sandwich - Layered document maker
 
+Apache::Request - Effective methods for dealing with client request data
+
 =head1 DESCRIPTION
 
 This bundle contains modules used by Apache mod_perl.
