@@ -569,6 +569,28 @@ void perl_module_init(server_rec *s, pool *p)
     perl_startup(s, p);
 }
 
+static void mod_perl_boot(void *data)
+{
+    /* make sure DynaLoader is loaded before XSLoader
+     * to workaround bug in 5.6.1 that can trigger a segv
+     * when using modperl as a dso
+     */
+    perl_require_module("DynaLoader", NULL);
+}
+
+static void mod_perl_xs_init(pTHX)
+{
+    xs_init(aTHX);
+
+    /* XXX: in 5.7.2+ we can call the body of mod_perl_boot here
+     * but in 5.6.1 the Perl runtime is not properly setup yet
+     * so we have to pull this stunt to delay
+     */
+#ifdef SAVEDESTRUCTOR_X
+    SAVEDESTRUCTOR_X(mod_perl_boot, 0);
+#endif
+}
+
 void perl_startup (server_rec *s, pool *p)
 {
     char *argv[] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
@@ -677,7 +699,7 @@ void perl_startup (server_rec *s, pool *p)
     MP_TRACE_g(fprintf(stderr, "constructing perl interpreter...ok\n"));
     perl_construct(perl);
 
-    status = perl_parse(perl, xs_init, argc, argv, NULL);
+    status = perl_parse(perl, mod_perl_xs_init, argc, argv, NULL);
     if (status != OK) {
 	MP_TRACE_g(fprintf(stderr,"not ok, status=%d\n", status));
 	perror("parse");
@@ -747,7 +769,7 @@ void perl_startup (server_rec *s, pool *p)
 #endif
 	
 #ifdef APACHE_PERL5LIB
-    perl_incpush(APACHE_PERL5LIB);
+    perl_inc_unshift(APACHE_PERL5LIB);
 #else
     av_push(GvAV(incgv), newSVpv(server_root_relative(p,""),0));
     av_push(GvAV(incgv), newSVpv(server_root_relative(p,"lib/perl"),0));
@@ -1388,6 +1410,15 @@ void perl_per_request_init(request_rec *r)
 	cfg->setup_env = 0; /* just once per-request */
     }
 
+    if (cfg->dir_env != cld->env) {
+        /* PerlSetEnv
+         * update only if the table changes across a request
+         */
+        MP_HASENV_on(cld);
+        mod_perl_dir_env(r, cld);
+        cfg->dir_env = cld->env;
+    }
+
     if(callbacks_this_request++ > 0) return;
 
     if (!r->main) {
@@ -1398,15 +1429,12 @@ void perl_per_request_init(request_rec *r)
 	(void)perl_request_rec(r);
     }
 
-    /* PerlSetEnv */
-    mod_perl_dir_env(r, cld);
-
     /* SetEnv PERL5LIB */
     if (!MP_INCPUSH(cld)) {
 	char *path = (char *)table_get(r->subprocess_env, "PERL5LIB");
 
 	if (path) {
-	    perl_incpush(path);
+           perl_inc_unshift(path);
 	    MP_INCPUSH_on(cld);
 	}
     }
