@@ -122,8 +122,13 @@ request_rec *sv2request_rec(SV *in, char *pclass, CV *cv)
 	    int klen = strlen(r_keys[i]);
 	    if(hv_exists((HV*)SvRV(in), r_keys[i], klen) &&
 	       (sv = *hv_fetch((HV*)SvRV(in), 
-			       r_keys[i], klen, FALSE)))
+			       r_keys[i], klen, FALSE))) {
+                if (SvROK(sv) && (SvTYPE(SvRV(sv)) == SVt_PVHV)) {
+                    /* dig deeper */
+                    return sv2request_rec(sv, pclass, cv);
+                }
 		break;
+            }
 	}
 	if(!sv)
 	    croak("method `%s' invoked by a `%s' object with no `r' key!",
@@ -421,8 +426,7 @@ void perl_run_endav(char *s)
     }
 }
 
-static I32
-errgv_empty_set(IV ix, SV* sv)
+static PERL_MG_UFUNC(errgv_empty_set, ix, sv)
 { 
     sv_setsv(sv, &sv_no);
     return TRUE;
@@ -606,6 +610,8 @@ array_header *perl_cgi_env_init(request_rec *r)
 
     add_common_vars(r); 
     add_cgi_vars(r); 
+    /* resetup global request rec, because it may set to an (invalid) subrequest by ap_add_cgi_vars */
+    perl_request_rec(r);
 
     if (!table_get(envtab, "TZ")) {
 	if ((tz = getenv("TZ")) != NULL) {
@@ -779,7 +785,7 @@ int perl_sv_is_http_code(SV *errsv, int *status)
 #define PERLLIB_SEP ':'
 #endif
 
-void perl_incpush(char *p)
+void perl_inc_unshift(char *p)
 {
     if(!p) return;
 
@@ -797,7 +803,8 @@ void perl_incpush(char *p)
 	    sv_setpv(libdir, p);
 	    p = Nullch;
 	}
-	av_push(GvAV(incgv), libdir);
+       av_unshift(GvAV(incgv), 1);
+       av_store(GvAV(incgv), 0, libdir);
     }
 }
 
