@@ -152,7 +152,7 @@ Benchmark: timing 10000 iterations of C, Perl...
   Perl: 21 secs (20.57 usr  0.14 sys = 20.71 cpu) 
 =cut
 
-my $date_str = "Sat, 18 Jul 1998 08:38:00 -0700";
+my $date_str = "Sat, 18 Jul 1998 08:38:00 GMT";
 
 test ++$i, Apache::Util::parsedate($date_str);
 
@@ -184,3 +184,7 @@ Benchmark: timing 10000 iterations of C, Perl...
   C:  1 secs ( 0.42 usr  0.00 sys =  0.42 cpu)
   Perl: 17 secs (16.76 usr  0.07 sys = 16.83 cpu)  
 =cut
+
+#my $hash = "aX9eP53k4DGfU";
+#test ++$i, Apache::Util::validate_password("dougm", $hash);
+#test ++$i, ! Apache::Util::validate_password("mguod", $hash);
